package org.xbib.groovy.crypt

import java.security.MessageDigest
import java.security.NoSuchAlgorithmException

/**
 * A enumeration of supported encryption types.
 */
enum CryptType {

    DES("0", "DES", DesCrypt),
    MD5("1", "MD5", Md5Crypt),
    /* Avoid the following:
      2 - the original BCrypt, which has been deprecated because of a security issue a long time before BCrypt became popular.
      2a - the official BCrypt algorithm and a insecure implementation in crypt_blowfish http://seclists.org/oss-sec/2011/q2/632
      2x - suggested for hashes created by the insecure algorithm for compatibility
      2y - suggested new marker for the fixed crypt_blowfish
     */
    SHA256("5", "SHA-256", Sha256Crypt),
    SHA512("6", "SHA-512", Sha512Crypt)

    private final Class<? extends Crypt> providerClass

    final String type

    final String algorithm

    private CryptType(String type, String algorithm, Class<? extends Crypt> providerClass) {
        this.type = type
        this.algorithm = algorithm
        this.providerClass = providerClass
    }

    /**
     * Gets the type instance that corresponds to the type specified by the
     * given salt.
     *
     * @param salt the subject salt
     * @return type instance
     * @throws NoSuchAlgorithmException if no type corresponds to the given salt
     */
    static CryptType forSalt(Salt salt) throws NoSuchAlgorithmException {
        for (CryptType type : values()) {
            if (type.type == salt.type) {
                return type
            }
        }
        throw new NoSuchAlgorithmException("unsupported type")
    }

    /**
     * Gets the {@code providerClass} property.
     *
     * @return
     */
    Class<? extends Crypt> getProviderClass() {
        providerClass
    }

    /**
     * Creates a new digest for the algorithm specified for this type.
     *
     * @return message digest
     * @throws NoSuchAlgorithmException
     */
    MessageDigest newDigest() throws NoSuchAlgorithmException {
        return MessageDigest.getInstance(algorithm)
    }
}
