package org.xbib.groovy.crypt

import java.nio.charset.StandardCharsets
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException

/**
 * An abstract base for password encryption that uses one of the SHA-2 
 * variants (SHA-256, SHA-512).
 */
abstract class Sha2Crypt extends Crypt {

    private static final String ROUNDS_PARAM = "rounds="

    private static final int MIN_ROUNDS = 1000

    private static final int MAX_ROUNDS = 999999999

    private static final int DEFAULT_ROUNDS = 5000

    private static final int MAX_SALT_LENGTH = 16

    protected Sha2Crypt(CryptType type) {
        super(type)
    }

    @Override
    protected String doCrypt(String password, Salt salt)
            throws NoSuchAlgorithmException, UnsupportedEncodingException {
        Integer rounds = rounds(salt)
        byte[] encrypted = doCrypt(password.getBytes(StandardCharsets.UTF_8),
                salt.getBytes(MAX_SALT_LENGTH, StandardCharsets.UTF_8),
                rounds == null ? DEFAULT_ROUNDS : rounds)
        passwordToString(encrypted, salt, MAX_SALT_LENGTH, rounds)
    }

    @Override
    protected String encodeParameters(Object... params) {
        params[0] == null ? null : ROUNDS_PARAM + params[0]
    }

    /**
     * Gets the number of rounds explicitly requested in the given salt
     * @param salt the subject salt
     * @return number of rounds requested or {@code null} if the salt does not
     *    specify the number of rounds
     */
    private static Integer rounds(Salt salt) {
        String params = salt.params
        if (params == null || !params.startsWith(ROUNDS_PARAM)) {
            return null
        }
        Integer rounds = Integer.valueOf(params.substring(ROUNDS_PARAM.length()))
        rounds = Math.max(MIN_ROUNDS, rounds)
        rounds = Math.min(rounds, MAX_ROUNDS)
        rounds
    }

    /**
     * Encrypts the given password.
     * @param password the password to encrypt
     * @param salt salt for the encryption
     * @param rounds number of rounds requested
     * @return byte array with encrypted value.
     * @throws NoSuchAlgorithmException
     */
    protected byte[] doCrypt(byte[] password, byte[] salt, int rounds)
            throws NoSuchAlgorithmException {
        final MessageDigest a = type.newDigest()
        final int digestLength = a.getDigestLength()
        a.update(password)
        a.update(salt)
        final MessageDigest b = type.newDigest()
        b.update(password)
        b.update(salt)
        b.update(password)
        final byte[] sumB = b.digest()
        int max = (password.length / digestLength) as int
        for (int i = 0; i < max; i++) {
            a.update(sumB)
        }
        a.update(sumB, 0, password.length % digestLength)
        for (int length = password.length; length > 0; length >>>= 1) {
            if ((length & 1) != 0) {
                a.update(sumB)
            } else {
                a.update(password)
            }
        }
        final byte[] sumA = a.digest()
        final MessageDigest dp = type.newDigest()
        for (int i = 0; i < password.length; i++) {
            dp.update(password)
        }
        final byte[] sumDP = dp.digest()
        final byte[] seqP = makeSequence(sumDP, password.length, digestLength)
        final MessageDigest ds = type.newDigest()
        max = 16 + ((int) sumA[0] & 0xff)
        for (int i = 0; i < max; i++) {
            ds.update(salt)
        }
        final byte[] sumDS = ds.digest()
        final byte[] seqS = makeSequence(sumDS, salt.length, digestLength)
        byte[] ac = sumA
        max = rounds
        for (int i = 0; i < max; i++) {
            final MessageDigest c = type.newDigest()
            if (i % 2 != 0) {
                c.update(seqP)
            } else {
                c.update(ac)
            }
            if (i % 3 != 0) {
                c.update(seqS)
            }
            if (i % 7 != 0) {
                c.update(seqP)
            }
            if (i % 2 != 0) {
                c.update(ac)
            } else {
                c.update(seqP)
            }
            ac = c.digest()
        }
        ac
    }

    /**
     * Makes a sequence as described in steps 16 and 20 of the algorithm.
     * @param sum the intermediate sum to place into the sequence
     * @param length length of the sequence in bytes
     * @param digestLength length of the digest in bytes
     * @return sequence
     */
    private static byte[] makeSequence(byte[] sum, int length, final int digestLength) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream()
        int max = (length / digestLength) as int
        for (int i = 0; i < max; i++) {
            outputStream.write(sum)
        }
        outputStream.write(sum, 0, length % digestLength)
        outputStream.toByteArray()
    }
}
