package org.xbib.groovy.crypt.random

import java.util.concurrent.locks.ReentrantLock

class MersenneTwisterRandom extends Random {

    private static final int SEED_SIZE_BYTES = 16
    private static final int N = 624
    private static final int M = 397
    private static final int[] MAG01 = [0, 0x9908b0df]
    private static final int UPPER_MASK = 0x80000000 as int
    private static final int LOWER_MASK = 0x7fffffff as int
    private static final int BOOTSTRAP_SEED = 19650218
    private static final int BOOTSTRAP_FACTOR = 1812433253
    private static final int SEED_FACTOR1 = 1664525
    private static final int SEED_FACTOR2 = 1566083941
    private static final int GENERATE_MASK1 = 0x9d2c5680 as int
    private static final int GENERATE_MASK2 = 0xefc60000 as int
    private static final SeedGenerator[] GENERATORS = [new SecureRandomSeedGenerator(), new DevRandomSeedGenerator()]
    private static final int BITWISE_BYTE_TO_INT = 0x000000FF
    private final ReentrantLock lock = new ReentrantLock()
    private final int[] mt = new int[N]
    private byte[] seed
    private int mtIndex = 0

    MersenneTwisterRandom() {
        this.seed = generateSeed(SEED_SIZE_BYTES)
        if (seed == null || seed.length != SEED_SIZE_BYTES) {
            throw new IllegalArgumentException("Mersenne Twister requires a 128-bit seed")
        }
        int[] seedInts = convertBytesToInts(this.seed)
        mt[0] = BOOTSTRAP_SEED
        for (mtIndex = 1; mtIndex < N; mtIndex++) {
            mt[mtIndex] = BOOTSTRAP_FACTOR * (mt[mtIndex - 1] ^ (mt[mtIndex - 1] >>> 30)) + mtIndex
        }
        int i = 1
        int j = 0
        for (int k = Math.max(N, seedInts.length); k > 0; k--) {
            mt[i] = (mt[i] ^ ((mt[i - 1] ^ (mt[i - 1] >>> 30)) * SEED_FACTOR1)) + seedInts[j] + j
            i++
            j++
            if (i >= N) {
                mt[0] = mt[N - 1]
                i = 1
            }
            if (j >= seedInts.length) {
                j = 0
            }
        }
        for (int k = N - 1; k > 0; k--) {
            mt[i] = (mt[i] ^ ((mt[i - 1] ^ (mt[i - 1] >>> 30)) * SEED_FACTOR2)) - i
            i++
            if (i >= N) {
                mt[0] = mt[N - 1]
                i = 1
            }
        }
        mt[0] = UPPER_MASK
    }

    @Override
    protected final int next(int bits) {
        int y = 0
        try {
            lock.lock()
            if (mtIndex >= N) {
                int kk
                for (kk = 0; kk < N - M; kk++) {
                    y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK)
                    mt[kk] = mt[kk + M] ^ (y >>> 1) ^ MAG01[y & 0x1]
                }
                for (; kk < N - 1; kk++) {
                    y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK)
                    mt[kk] = mt[kk + (M - N)] ^ (y >>> 1) ^ MAG01[y & 0x1]
                }
                y = (mt[N - 1] & UPPER_MASK) | (mt[0] & LOWER_MASK)
                mt[N - 1] = mt[M - 1] ^ (y >>> 1) ^ MAG01[y & 0x1]
                mtIndex = 0
            }
            y = mt[mtIndex++]
        }
        finally {
            lock.unlock()
        }
        y ^= (y >>> 11)
        y ^= (y << 7) & GENERATE_MASK1
        y ^= (y << 15) & GENERATE_MASK2
        y ^= (y >>> 18)
        return y >>> (32 - bits)
    }

    private static byte[] generateSeed(int length) {
        for (SeedGenerator generator : GENERATORS) {
            try {
                return generator.generateSeed(length)
            }
            catch (SeedException ex) {
                // ignore
            }
        }
        throw new IllegalStateException("no seed generator available")
    }

    private static int[] convertBytesToInts(byte[] bytes) {
        int l = bytes.length
        if (l % 4 != 0) {
            throw new IllegalArgumentException("number of input bytes must be a multiple of 4")
        }
        int size = l.intdiv(4)
        int[] ints = new int[size]
        for (int i = 0; i < ints.length; i++) {
            ints[i] = convertBytesToInt(bytes, i * 4)
        }
        return ints
    }

    private static int convertBytesToInt(byte[] bytes, int offset) {
        (BITWISE_BYTE_TO_INT & bytes[offset + 3]) | ((BITWISE_BYTE_TO_INT & bytes[offset + 2]) << 8) | ((BITWISE_BYTE_TO_INT & bytes[offset + 1]) << 16) | ((BITWISE_BYTE_TO_INT & bytes[offset]) << 24)
    }
}