package org.xbib.groovy.crypt.random

import java.nio.charset.StandardCharsets
import java.util.concurrent.atomic.AtomicInteger

/**
 * UUID generator. The UUIDs are essentially flake ids
 * but we use 6 (not 8) bytes for timestamp and 3 (not 2) bytes for sequence number.
 */
class UUIDGenerator {

    private final static byte[] secureAddress = getSecureAddress()

    private final AtomicInteger sequenceNumber = new AtomicInteger(RandomUtil.secureRandom.nextInt())

    private long lastTimestamp

    String getBase16UUID()  {
        StringBuilder sb = new StringBuilder()
        getUUIDBytes().each {
            sb.append(Integer.toHexString((int) it & 0xFF))
        }
        sb.toString()
    }

    String getBase64UUID() {
        byte[] encoded = getBase64UUIDBytes()
        new String(encoded, 0, encoded.length, StandardCharsets.US_ASCII)
    }

    private static void putLong(byte[] array, long l, int pos, int numberOfLongBytes) {
        for (int i=0; i<numberOfLongBytes; ++i) {
            array[pos + numberOfLongBytes-i-1] = (byte) (l >>> (i*8))
        }
    }

    private static byte[] getSecureAddress() {
        byte[] address = null
        try {
            address = getMacAddress()
        } catch (Throwable t) {
        }
        if (!isValidAddress(address)) {
            address = constructDummyMulticastAddress()
        }
        byte[] bytes = new byte[6]
        RandomUtil.secureRandom.nextBytes(bytes)
        for (int i = 0; i < 6; ++i) {
            bytes[i] = (bytes[i] ^ address[i]) as byte
        }
        bytes
    }

    private static byte[] constructDummyMulticastAddress() {
        byte[] bytes = new byte[6]
        RandomUtil.secureRandom.nextBytes(bytes)
        bytes[0] = (bytes[0] | 0x01) as byte
        bytes
    }

    private static byte[] getMacAddress() throws SocketException {
        Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces()
        if (en != null) {
            while (en.hasMoreElements()) {
                NetworkInterface nint = en.nextElement()
                if (!nint.isLoopback()) {
                    byte[] address = nint.getHardwareAddress()
                    if (isValidAddress(address)) {
                        return address
                    }
                }
            }
        }
        null
    }

    private static boolean isValidAddress(byte[] address) {
        if (address == null || address.length != 6) {
            return false
        }
        for (byte b : address) {
            if (b != 0x00 as byte) {
                return true
            }
        }
        false
    }

    private byte[] getUUIDBytes() {
        int sequenceId = sequenceNumber.incrementAndGet() & 0xffffff
        long timestamp = System.currentTimeMillis()
        synchronized (this) {
            timestamp = Math.max(lastTimestamp, timestamp)
            if (sequenceId == 0) {
                timestamp++
            }
            lastTimestamp = timestamp
        }
        byte[] uuidBytes = new byte[15]
        putLong(uuidBytes, timestamp, 0, 6)
        System.arraycopy(secureAddress, 0, uuidBytes, 6, secureAddress.length)
        putLong(uuidBytes, sequenceId, 12, 3)
        uuidBytes
    }

    private byte[] getBase64UUIDBytes()  {
        byte[] uuidBytes = getUUIDBytes()
        byte[] encoded
        try {
            encoded = Base64.encoder.encode(uuidBytes)
        } catch (IOException e) {
            throw new IllegalStateException("should not be thrown", e)
        }
        encoded
    }
}
